import os
from pkg_resources import resource_string
from markdown2 import Markdown
from pystache import Renderer
import emoji

def md_to_html(in_file, out_file,
               template_kargs):
    with open(in_file, 'r') as f_in:
        markdowner = Markdown()
        html_str = markdowner.convert(f_in.read())
        html_emojized_str = emoji.emojize(html_str, use_aliases=True)
        template_kargs['requirements_html'] = html_emojized_str
    renderer = Renderer()
    template = resource_string(__name__,  'REQUIREMENTS.html.template')
    with open(out_file, 'w') as f_out:
        html_str = renderer.render(template, template_kargs)
        f_out.write(html_str)
    return out_file




import os
import logging
from mdutils import MdUtils
from pds_github_util.corral.herd import Herd

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

COLUMNS = ['download', 'manual', 'changelog', 'requirements', 'license', 'feedback']

def write_build_summary(gitmodules=None, root_dir='.', output_file_name=None, token=None, dev=False, version=None):

    herd = Herd(gitmodules=gitmodules, dev=dev, token=token)

    if version is None:
        version = herd.get_shepard_version()
    else:
        # for unit test
        herd.set_shepard_version(version)

    logger.info(f'build version is {version}')
    if dev and not ('dev' in version or 'SNAPSHOT' in version):
        logger.error("version of build does not contain dev or SNAPSHOT, dev build summary is not generated")
        exit(1)
    elif not dev and ('dev' in version or 'SNAPSHOT' in version):
        logger.error("version of build contains dev or SNAPSHOT, release build summary is not generated")
        exit(1)

    if not output_file_name:
        output_file_name = os.path.join(root_dir, version, 'index')
    os.makedirs(os.path.dirname(output_file_name), exist_ok=True)

    software_summary_md = MdUtils(file_name=output_file_name, title=f'Software Summary (build {version})')

    column_headers = []
    for column in COLUMNS:
        column_headers.append(f'![{column}](https://nasa-pds.github.io/pdsen-corral/images/{column}_text.png)')

    table = ["tool", "version", "description", *column_headers]
    n_columns = len(table)
    for k, ch in herd.get_cattle_heads().items():
        table.extend(ch.get_table_row())
    software_summary_md.new_table(columns=n_columns,
                                  rows=herd.number_of_heads()+1,
                                  text=table,
                                  text_align='center')

    software_summary_md.create_md_file()

    return herd



import github3
import git
from packaging import version



class Tags:
    def __init__(self, org, repo, token=None):
        self._organization = org
        self._repository = repo

        gh = github3.login(token=token)
        self._repo = gh.repository(self._organization, self._repository)

        self.sorted_tags = []
        for tag in self._repo.tags():
            commit = self._repo.commit(tag.commit.sha)
            self.sorted_tags.append({'date': commit.as_dict()['commit']['author']['date'], 'name': tag.name})

        self.sorted_tags.sort(key=lambda t: t['date'])

    def get_earliest_tag_after(self, date_iso):
        for tag in self.sorted_tags:
            if tag['date'] > date_iso:
                return tag['name']

    def get_latest_tag(self, dev=False):
        latest_tag = None
        for tag in self._repo.tags():
            if Tags.is_dev_version(tag.name) and dev:  # if we have a dev version and we look for dev version
                latest_tag = Tags.get_max_tag(tag.name, latest_tag) if latest_tag else tag.name
            elif not (Tags.is_dev_version(tag.name) or dev):  # if we don't have a dev version and we look for stable version
                latest_tag = Tags.get_max_tag(tag.name, latest_tag) if latest_tag else tag.name

        return latest_tag.__str__() if latest_tag else None

    @staticmethod
    def is_dev_version(tag_name):
        return tag_name.endswith("-dev") or tag_name.endswith("-SNAPSHOT")

    @staticmethod
    def get_max_tag(tag_name, other_tag_name):
        vers = version.parse(tag_name)
        other_vers = version.parse(other_tag_name)
        return tag_name if (vers > other_vers) else other_tag_name






import os
from configparser import ConfigParser
from datetime import datetime
import logging
from pds_github_util.corral import CattleHead

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class Herd:
    def __init__(self, gitmodules=None, dev=False, token=False):
        self._dev =dev
        self._token = token
        self._config = ConfigParser()
        if gitmodules:
            self._config.read(gitmodules)
        else:
            self._config.read(os.path.join(os.getcwd(), ".gitmodules"))

        self._gather_the_herd()

    def number_of_heads(self):
        return len(self._herd)

    def get_cattle_heads(self):
        return self._herd

    def _gather_the_herd(self):
        logger.info('gather the herd of submodules listed in .gitmodules')

        self._herd = {}
        self._shepard_version = None
        for section in self._config.sections():
            if 'submodule "."' not in section:
                module_array = section.split(" ")
                if len(module_array) >= 2:
                    module_name = module_array[1].strip('"')
                else:
                    logger.error(f'section {section} is malformed, expected format is: [submodule "<module name>"]')

                optional_module_options = {k:self._config.get(section, k).strip("/") for k in ['version'] if self._config.has_option(section, k)}
                self._herd[module_name] = CattleHead(module_name,
                                                     self._config.get(section, "url").strip("/"),
                                                     dev=self._dev,
                                                     token=self._token,
                                                     **optional_module_options)
            else:
                self._shepard_version = self._config.get(section, 'version')
                self._release_date = datetime.fromisoformat(self._config.get(section, 'release'))
                self._update_date = datetime.now()

        return 0

    def set_shepard_version(self, version):
        """
        For unit test purpose
        :param version:
        :return:
        """
        self._config['submodule "."']['version'] = version

    def get_shepard_version(self):
        return self._config.get('submodule "."', 'version').strip(" ")

    def get_release_datetime(self):
        return self._release_date

    def get_update_datetime(self):
        return self._update_date


import os
import logging
import shutil
import re
from git import Repo
import github3

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)



def loop_checkout_on_branch(repo_full_name, branch_regex, callback, token=None, local_git_tmp_dir='/tmp'):
    repo_full_name_array = repo_full_name.split("/")
    org = repo_full_name_array[0]
    repo_name = repo_full_name_array[1]

    gh = github3.login(token=token)
    gh_repo = gh.repository(org, repo_name)

    prog = re.compile(branch_regex)

    local_path = os.path.join(local_git_tmp_dir, repo_name)
    if os.path.exists(local_path):
      shutil.rmtree(local_path)

    for branch in gh_repo.branches():
        if prog.match(branch.name):
            logger.info(f'branch {branch.name}')
            remote_url = gh_repo.git_url.replace('git://', f'https://{token}:x-oauth-basic@')
            g_repo = clone_checkout_branch(remote_url, local_path, branch.name)
            yield callback()



def ping_repo_branch(repo_full_name, branch, message, token=None):
    repo_full_name_array = repo_full_name.split("/")
    org = repo_full_name_array[0]
    repo_name = repo_full_name_array[1]

    gh = github3.login(token=token)
    gh_repo = gh.repository(org, repo_name)

    remote_url = gh_repo.git_url.replace('git://', f'https://{token}:x-oauth-basic@')
    g_repo = clone_checkout_branch(remote_url, os.path.join('/tmp', repo_name), branch)
    g_repo.index.commit(message)
    origin = g_repo.remote(name='origin')
    origin.push()



def clone_checkout_branch(git_remote_url, local_repo, branch):
    os.makedirs(local_repo, exist_ok=True)
    repo = Repo.init(local_repo)
    if len(repo.remotes) == 0 or 'origin' not in [r.name for r in repo.remotes]:
        repo.create_remote('origin', git_remote_url)
    repo.git.fetch()
    repo.git.checkout(branch)
    repo.git.pull()
    return repo





import os
import logging
from configparser import ConfigParser
import github3

from pds_github_util.corral.cattle_head import is_dev_version, get_max_tag

logger = logging.getLogger(__name__)
logger.setLevel("INFO")


def grab_latest_versions(token, dev=False):

    config = ConfigParser()
    config.read(os.path.join(os.getcwd(), ".gitmodules"))

    latest_versions = {}

    for section in config.sections():
        submodule_name = section.split(" ")[1].strip('"')
        github_path = config.get(section, "url").strip("/")
        org = github_path.split("/")[-2]
        repo_name =  github_path.split("/")[-1]  # use submodule_name would make no difference but I feel like I better trust the value in the url
        gh = github3.login(token=token)
        repo = gh.repository(org, repo_name)
        latest_tag = None
        for tag in repo.tags():
            if is_dev_version(tag.name) and dev: # if we have a dev version and we look for dev version
                latest_tag = get_max_tag(tag.name, latest_tag) if latest_tag else tag.name
            elif not (is_dev_version(tag.name) or dev): # if we don't have a dev version and we look for stable version
                latest_tag = get_max_tag(tag.name, latest_tag) if latest_tag else tag.name
        if latest_tag:
            latest_versions[repo_name] = latest_tag.__str__()

    return latest_versions




import unittest
import os
import logging
from datetime import datetime
from pds_github_util.tags.tags import Tags

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

GITHUB_TOKEN = os.environ.get("GITHUB_TOKEN")


class MyTestCase(unittest.TestCase):
    def test_get_earliest_tag_after(self):
        tags = Tags('NASA-PDS', 'validate', token=GITHUB_TOKEN)

        tags.get_earliest_tag_after(datetime(2020, 1, 1).isoformat().replace('+00:00', 'Z'))


if __name__ == '__main__':
    unittest.main()



import argparse
from pds_github_util.branches.git_actions import ping_repo_branch

def main():
    parser = argparse.ArgumentParser(description='empty commit on a repo branch')
    parser.add_argument('--repo', dest='repo',
                        help='repostory full name with owner, e.g. nasa-pds/pdsen-corral')
    parser.add_argument('--token', dest='token',
                        help='github personal access token')
    parser.add_argument('--branch', dest='branch',
                        help='branch name')
    parser.add_argument('--message', dest='message',
                        help='commit message')
    args = parser.parse_args()

    # read organization and repository name
    ping_repo_branch(args.repo, args.branch, args.message, token=args.token)


if __name__ == '__main__':
    main()


import argparse

from pds_github_util.output.summary import write_build_summary


def main():
    parser = argparse.ArgumentParser(description='Create new snapshot release')
    parser.add_argument('--output', dest='output',
                        default='output',
                        help='markdown output file name')
    parser.add_argument('--token', dest='token',
                        help='github personal access token')
    parser.add_argument('--dev', dest='dev',
                        action='store_true',
                        default=False,
                        help='if present we search for dev versions, otherwise stable versions are returned')
    args = parser.parse_args()

    output_dir = write_build_summary(root_dir=args.output, token=args.token, dev=args.dev)
    print(output_dir)




if __name__ == "__main__":
    main()




import os
import mdutils
from datetime import datetime

SEMANTIC_VERSION_REGEX = r'\d+\.\d+(-SNAPSHOT)?'


def colored_datetime(d, colored=True, format='%Y-%m-%d'):
    d_str = d.strftime(format)
    if colored:
        color = 'orange' if d > datetime.now() else 'green'
        return f'<span style="color:{color}">{d_str}</span>'
    else:
        return d_str


class HerdTable:
    def __init__(self, columns):
        self.table = columns
        self.n_rows = 1
        self.n_columns = len(columns)

    def add_herd(self, h):
        v = h.get_shepard_version()
        build_link = f'[{v}](./{v})'
        self.table.extend([build_link,
                           colored_datetime(h.get_release_datetime()),
                           colored_datetime(h.get_update_datetime(), colored=False)])
        self.n_rows += 1

    def __len__(self):
        return self.n_rows - 1

    def write_to_md_file(self, md_file):
        md_file.new_table(columns=self.n_columns,
                          rows=self.n_rows,
                          text=self.table,
                          text_align='center')


def update_index(root_dir, herds):
    index_file_name = os.path.join(root_dir, 'index.md')
    index_md_file = mdutils.MdUtils(file_name=index_file_name, title=f'PDS Engineering Node software suite, builds')

    herds.sort(key=lambda x: x.get_release_datetime(), reverse=True)
    herds_iterator = iter(herds)

    now = datetime.now()

    # dev/uit releases
    table_development_releases = HerdTable(["build", "planned release", "update"])
    while True:
        herd = next(herds_iterator)
        if herd.get_release_datetime() > now:
            table_development_releases.add_herd(herd)
        else:
            break

    # stable release
    table_latest_stable_release = HerdTable(["build", "release", "update"])
    table_latest_stable_release.add_herd(herd)

    # archived releases
    table_archived_releases = HerdTable(["build", "release", "update"])
    while True:
        try:
            herd = next(herds_iterator)
            table_archived_releases.add_herd(herd)
        except StopIteration as e:
            break

    if len(table_latest_stable_release):
        index_md_file.new_paragraph("Latest stable release:")
        index_md_file.new_line('')
        table_latest_stable_release.write_to_md_file(index_md_file)

    if len(table_development_releases):
        index_md_file.new_line('')
        index_md_file.new_paragraph("Development releases:")
        index_md_file.new_line('')
        table_development_releases.write_to_md_file(index_md_file)

    if len(table_archived_releases):
        index_md_file.new_line('')
        index_md_file.new_paragraph("Archived stable releases:")
        index_md_file.new_line('')
        table_archived_releases.write_to_md_file(index_md_file)

    img = index_md_file.new_inline_image('new PDS logo test', 'https://nasa-pds.github.io/pdsen-corral/images/logo.png')
    index_md_file.new_line(img)

    index_md_file.create_md_file()




import unittest
import os
from pds_github_util.gh_pages.build_summaries import build_summaries

TOKEN = os.environ.get('GITHUB_TOKEN')

class MyTestCase(unittest.TestCase):
    def test_something(self):
        build_summaries(TOKEN)


if __name__ == '__main__':
    unittest.main()



'''
LDDTool Wrapper

Tool that downloads PDS Local Data Dictionary Tool (LDDTool) and executes based upon
input arguments.
'''
import argparse
import glob
import logging
import os
import requests
import shutil
import sys
import traceback

from datetime import datetime
from subprocess import Popen, CalledProcessError, PIPE, STDOUT

from pds_github_util.tags.tags import Tags
from pds_github_util.assets.assets import download_asset, unzip_asset

GITHUB_ORG = 'NASA-PDS'
GITHUB_REPO = 'pds4-information-model'

# LDDTool defaults
LDDTOOL_DEFAULT_ARGS = '-lpJ'

# Schema download constants
PDS_SCHEMA_URL = 'https://pds.nasa.gov/pds4/pds/v1/'
PDS_DEV_SCHEMA_URL = 'https://pds.nasa.gov/datastandards/schema/develop/pds/'
DOWNLOAD_PATH = '/tmp'

# Quiet github3 logging
logger = logging.getLogger('github3')
logger.setLevel(level=logging.WARNING)

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def convert_pds4_version_to_alpha(pds4_version):
    pds4_version_short = ''
    version_list = pds4_version.split('.')
    for num in version_list:
        if int(num) >= 10:
            pds4_version_short += chr(ord('@') + (int(num) % 10 + 1))
        else:
            pds4_version_short += num

    return pds4_version_short


def find_dependency_ingest_ldds(ingest_ldd_src_dir):
    # Get any dependencies first
    dependencies_path = os.path.join(ingest_ldd_src_dir, 'dependencies')
    dependency_ldds = glob.glob(os.path.join(dependencies_path, '*', 'src', '*IngestLDD*.xml'))
    return dependency_ldds


def find_primary_ingest_ldd(ingest_ldd_src_dir):
    ingest_ldd = glob.glob(os.path.join(ingest_ldd_src_dir, '*IngestLDD*.xml'))
    return ingest_ldd


def get_latest_release(token, dev=False):
    _tags = Tags(GITHUB_ORG, GITHUB_REPO, token=token)
    return _tags._repo.release_from_tag(_tags.get_latest_tag(dev=dev))


def prep_ldd_output_path(ldd_output_path):
    # Crawl two directories up and remove everything
    parent_dir = os.path.dirname(os.path.dirname(ldd_output_path.rstrip(os.sep)))
    # parent_dir = ldd_output_path

    logger.info(f'Cleaning up {parent_dir}')
    for path, dirs, files in os.walk(parent_dir):
        for f in files:
            filepath = os.path.join(path, f)
            logger.info(f'Removing {filepath}')
            os.remove(filepath)

    if not os.path.exists(ldd_output_path):
        os.makedirs(ldd_output_path)


def exec_lddtool(executable, execution_cwd, args, ingest_ldds, log_path=os.path.expanduser('~')):
    dtime = datetime.now().strftime('%Y%m%d%H%M%S')
    log_out = os.path.join(log_path, f'lddtool_run_log_{dtime}.txt')
    if not os.path.exists(os.path.dirname(log_out)):
        os.makedirs(os.path.dirname(log_out))

    cmd = ['bash', executable ]
    args.extend(ingest_ldds)
    cmd.extend(args)
    logger.info(cmd)

    with Popen(cmd, cwd=execution_cwd, stdout=PIPE, stderr=STDOUT, bufsize=1, universal_newlines=True) as p:
        with open(log_out, 'w') as f:
            for line in p.stdout:
                print(line, end='') # process line here
                f.write(line)

    if p.returncode != 0:
        raise CalledProcessError(p.returncode, p.args)


def main():

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
        description=__doc__)

    parser.add_argument('--deploy_dir',
                        help='directory to deploy the validate tool on the file system',
                        default='/tmp')
    parser.add_argument('--ldd_output_path',
                        help='directory to output generated LDDs',
                        required=True)
    parser.add_argument('--ingest_ldd_src_dir',
                        help='/path/to/src/ dir for IngestLDD file',
                        default=os.path.join('tmp', 'logs'),
                        required=True)
    parser.add_argument('--token',
                        help='github token')
    parser.add_argument('--output_log_path',
                        help='path(s) to output validate run log file',
                        default=os.path.join('tmp', 'logs'))
    parser.add_argument('--with_pds4_version',
                        help=('force the following PDS4 version. software will '
                              'download and validate with this version of the '
                              'PDS4 Information Model. this version should be '
                              'the semantic numbered version. e.g. 1.14.0.0'))
    parser.add_argument('--use_lddtool_unstable',
                        help=('force the use of the latest unstable LDDTool release. '
                              'by default, uses latest stable release'),
                        action='store_true', default=False)

    args = parser.parse_args()

    token = args.token or os.environ.get('GITHUB_TOKEN')

    if not token:
        logger.error(f'Github token must be provided or set as environment variable (GITHUB_TOKEN).')
        sys.exit(1)

    try:

        lddtool_args = [ LDDTOOL_DEFAULT_ARGS ]

        if args.with_pds4_version:
            lddtool_args.extend(['-V', convert_pds4_version_to_alpha(args.with_pds4_version)])

        # Get the IngestLDDs
        # lddtool_args.extend()

        # cleanup the LDD Output area before generating LDDs
        prep_ldd_output_path(args.ldd_output_path)
        
        pkg = download_asset(get_latest_release(token, dev=args.use_lddtool_unstable), 
                                                args.deploy_dir, file_extension='.zip')
        sw_dir = unzip_asset(pkg, args.deploy_dir)

        # Generate dependency LDDs
        ingest_ldds = find_dependency_ingest_ldds(args.ingest_ldd_src_dir)
        for ingest in ingest_ldds:
            # execute LDDTool
            exec_lddtool(os.path.join(sw_dir, 'bin', 'lddtool'), args.ldd_output_path, lddtool_args, [ ingest ], log_path=args.output_log_path)

        # Generate final LDDs
        ingest_ldds.extend(find_primary_ingest_ldd(args.ingest_ldd_src_dir))

        # execute LDDTool
        exec_lddtool(os.path.join(sw_dir, 'bin', 'lddtool'), args.ldd_output_path, lddtool_args, ingest_ldds, log_path=args.output_log_path)

    except CalledProcessError:
        logger.error(f'FAILED: LDDTool failed unexpectedly. See output logs.')
        sys.exit(1)
    except Exception as e:
        traceback.print_exc()
        sys.exit(1)

    logger.info(f'SUCCESS: LDD Generation complete.')

if __name__ == '__main__':
    main()



import os
import logging
import argparse
from pkg_resources import resource_filename
from functools import partial
from shutil import copytree, rmtree, copy
from pds_github_util.branches.git_actions import loop_checkout_on_branch
from pds_github_util.gh_pages.summary import write_build_summary
from pds_github_util.gh_pages.root_index import update_index


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

def copy_resources():
    logger.info("write static resources (img, config)...")
    resources = resource_filename(__name__, 'resources')
    for f in os.listdir(resources):
        i_p = os.path.join(resources, f)
        o_p = os.path.join(os.getcwd(), f)
        if os.path.isdir(i_p):
            if os.path.exists(o_p):
                rmtree(o_p, ignore_errors=True)
            copytree(i_p, o_p)
        else:
            if os.path.exists(o_p):
                os.remove(o_p)
            copy(i_p, os.getcwd())



def build_summaries(token):
    copy_resources()

    herds = []
    # loop on stable releases
    for herd in loop_checkout_on_branch('NASA-PDS/pdsen-corral',
                            '[0-9]+\.[0-9]+',
                            partial(write_build_summary,
                                    root_dir=os.getcwd(),
                                    gitmodules='/tmp/pdsen-corral/.gitmodules',
                                    token=token,
                                    dev=False),
                            token=token,
                            local_git_tmp_dir='/tmp'):
        herds.append(herd)

    # dev release on master
    herd = next(loop_checkout_on_branch('NASA-PDS/pdsen-corral',
                            'master',
                            partial(write_build_summary,
                                    root_dir=os.getcwd(),
                                    gitmodules='/tmp/pdsen-corral/.gitmodules',
                                    token=token,
                                    dev=True),
                            token=token,
                            local_git_tmp_dir='/tmp'))

    herds.append(herd)

    update_index(os.getcwd(), herds)


def main():
    parser = argparse.ArgumentParser(description='Create new snapshot release')
    parser.add_argument('--token', dest='token',
                        help='github personal access token')
    args = parser.parse_args()
    build_summaries(args.token)



if __name__ == "__main__":
    main()




import os
import unittest
from pds_github_util.corral.cattle_head import CattleHead

TOKEN = os.environ.get('GITHUB_TOKEN')

class MyTestCase(unittest.TestCase):
    def test_get_changelog_signet(self):
        cattle_head = CattleHead("validate", "https://github.com/nasa-pds/validate", "validate PDS formats", token=TOKEN)
        changelog_signet = cattle_head._get_changelog_signet()


if __name__ == '__main__':
    unittest.main()



import os
import logging
import github3
import re
from mdutils import MdUtils
from pds_github_util.tags.tags import Tags

from pds_github_util.html.md_to_html import md_to_html

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class NoAppropriateVersionFoundException(Exception):
    pass

class Requirements:
    ISSUE_TYPES = ['bug', 'enhancement']

    def __init__(self, org, repo, token=None, dev=False):
        self._organization = org
        self._repository = repo
        self._dev = dev

        gh = github3.login(token=token)
        self._repo = gh.repository(self._organization, self._repository)
        self._requirements = self._get_requirements_from_issues()
        self._tags = Tags(org, repo, token=token)
        self._current_tag = self._tags.get_latest_tag(self._dev)
        self._requirements_tag_map = self._get_requirement_tag_map()


    def _get_requirement_topic(self, issue):
        requirement_topic = None
        for label in issue.labels():
            if label.name.startswith("requirement-topic"):
                requirement_topic = label.name.split(':')[1]
                continue

        if not requirement_topic:
            requirement_topic = 'default'

        return requirement_topic




    def _get_requirements_from_issues(self):
        summary = {}

        for issue in self._repo.issues(state='all', direction='asc', labels='requirement'):
            requirement_topic = self._get_requirement_topic(issue)
            if requirement_topic not in summary.keys():
                summary[requirement_topic] = []
            summary[requirement_topic].append({'number': issue.number,
                                               'title': issue.title,
                                               'link': issue.url})

        return summary

    def _get_requirement_tag_map(self):

        requirements_issue_map = {}
        requirements_tag_map = {}
        for issue in self._repo.issues(state='closed', direction='asc'):
            body_sections = issue.body.split("**Applicable requirements")
            if len(body_sections) > 1:
                impacted_requirements_str = body_sections[1]
                prog = re.compile("#[0-9]+")
                requirements = prog.findall(impacted_requirements_str)
                requirements = [ int(req[1:]) for req in requirements] # remove leading # and convert to int to be consistent with requirement dictionnary
                for req in requirements:
                    if req not in requirements_tag_map.keys():
                        requirements_tag_map[req] = {'issues': set(),  'tags': set()}
                    issue_date_isoz = issue.closed_at.isoformat().replace('+00:00', 'Z')
                    earliest_tag_closed_after = self._tags.get_earliest_tag_after(issue_date_isoz)
                    requirements_tag_map[req]['issues'].add(issue.number)
                    requirements_tag_map[req]['tags'].add(earliest_tag_closed_after)
        return requirements_tag_map

    def get_requirements(self):
        return self._requirements

    @staticmethod
    def _version_paragraph_intro(versions_len):
        if versions_len == 0:
            return ''
        elif versions_len ==1:
            return 'The version implementing or impacting this requirement is:'
        else:
            return 'The versions implementing or impacting this requirement are:'


    @staticmethod
    def _issue_is_bug_or_enhancement(issue):
        for label in issue.labels():
            if label.name in Requirements.ISSUE_TYPES:
                return label.name



    def write_requirements(self, root_dir='.', md_file_name=None, format='md'):
        if not md_file_name:
            if self._current_tag:
                md_file_name = os.path.join(root_dir, self._current_tag, 'REQUIREMENTS.md')
            else:
                dev_or_stable = "dev" if self._dev else "stable"
                raise NoAppropriateVersionFoundException("No suitable version for " + dev_or_stable + "release")

        os.makedirs(os.path.dirname(md_file_name), exist_ok=True)
        requirements_md = MdUtils(file_name=md_file_name, title="Requirements Summary")

        for req_topic in self._requirements:
            requirements_md.new_header(level=1, title=req_topic)
            for req in self._requirements[req_topic]:
                impacted = self._current_tag in self._requirements_tag_map[req['number']]['tags'] if req['number'] in self._requirements_tag_map else False
                impacted_icon = ':boom:' if impacted else ''
                title = f"{req['title']} ([#{req['number']}](https://github.com/{self._repo}/issues/{req['number']})) {impacted_icon}"
                requirements_md.new_header(level=2, title=title)
                if impacted:
                    issue_lines = {t : [] for t in Requirements.ISSUE_TYPES}
                    for n in self._requirements_tag_map[req['number']]['issues']:
                        issue = self._repo.issue(n)
                        bug_or_enhancement = Requirements._issue_is_bug_or_enhancement(issue)
                        issue_lines[bug_or_enhancement].append(f'{issue.title} ([#{n}](https://github.com/{self._repo}/issues/{n}))')

                    for issue_type, issue_list in issue_lines.items():
                        if len(issue_lines[issue_type]):
                            requirements_md.new_paragraph(f'The {issue_type}s which impact this requirements are:')
                            requirements_md.new_list(issue_list)

                else:
                    requirements_md.new_paragraph('This requirement is not impacted by the current version')

        requirements_md.create_md_file()
        if format == 'md':
            return md_file_name
        if format == 'html':
            html_file_name = md_file_name.replace('.md', '.html')
            return md_to_html(md_file_name, html_file_name,
                              {'name': self._repo, 'description': self._repo.description, 'tag': self._current_tag})
        else:
            logger.error(f'output format {format} is not supported')
            return ''




import unittest
import os
import logging
from pds_github_util.corral.herd import Herd
from pds_github_util.gh_pages.summary import write_build_summary

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

TOKEN = os.environ.get('GITHUB_TOKEN')

class MyTestCase(unittest.TestCase):

    gitmodules = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                              '.gitmodules')

    def test_gather_the_herd(self):
        herd = Herd(gitmodules=self.gitmodules,
                    token=TOKEN)
        cattle_heads = herd.get_cattle_heads()
        version = herd.get_shepard_version()

    def test_summary_dev(self):
        write_build_summary(gitmodules=self.gitmodules,
                            output_file_name='output/dev_summary.md',
                            token=TOKEN, dev=True,
                            version='10.0-SNAPSHOT')

    def test_summary_release(self):
        write_build_summary(gitmodules=self.gitmodules,
                            output_file_name='output/rel_summary.md',
                            token=TOKEN, dev=False,
                            version='10.0')


if __name__ == '__main__':
    unittest.main()



import argparse
import logging
import os
import sys

from zipfile import ZipFile

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def download_asset(release, output_path, file_extension=None):
    for _asset in release.assets():
        # print(a.name)
        if (_asset.name.endswith(file_extension)):
            logger.info(f'Downloading asset {_asset.name}')
            _output = os.path.join(output_path, _asset.name)
            _asset.download(path=_output)
            break

    return _output


def unzip_asset(inzip, output_path):
    if not os.path.exists(inzip):
        raise Exception(f'Github Asset "{inzip}" not found. Check release.')

    if not os.path.exists(output_path):
        raise Exception(f'Output path "{output_path}" not found.')

    logger.info(f'Unzipping asset {inzip}')
    with ZipFile(inzip, 'r') as _zip_ref:
        _zip_ref.extractall(output_path)
        _outdir = os.path.join(output_path, _zip_ref.namelist()[0])

    return _outdir


'''
PDS Validate Wrapper

Tool that downloads PDS Validate Tool and executes based upon
input arguments.
'''
import argparse
import glob
import logging
import os
import requests
import sys
import traceback

from datetime import datetime
from subprocess import Popen, CalledProcessError, PIPE, STDOUT
from zipfile import ZipFile

from pds_github_util.tags.tags import Tags
from pds_github_util.assets.assets import download_asset, unzip_asset
from pds_github_util.utils.ldd_gen import convert_pds4_version_to_alpha

GITHUB_ORG = 'NASA-PDS'
GITHUB_REPO = 'validate'

# Schema download constants
PDS_SCHEMA_URL = 'https://pds.nasa.gov/pds4/pds/v1/'
PDS_DEV_SCHEMA_URL = 'https://pds.nasa.gov/datastandards/schema/develop/pds/'
DOWNLOAD_PATH = '/tmp'

# Quiet github3 logging
logger = logging.getLogger('github3')
logger.setLevel(level=logging.WARNING)

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def get_latest_release(token, dev=False):
    _tags = Tags(GITHUB_ORG, GITHUB_REPO, token=token)
    return _tags._repo.release_from_tag(_tags.get_latest_tag(dev=dev))


def exec_validate(executable, args, log_path=os.path.expanduser('~')):
    dtime = datetime.now().strftime('%Y%m%d%H%M%S')
    log_out = os.path.join(log_path, f'validate_report_{dtime}.txt')
    if not os.path.exists(os.path.dirname(log_out)):
        os.makedirs(os.path.dirname(log_out))

    cmd = ['bash', executable ]
    # print(args)
    cmd.extend(args)
    with Popen(cmd, stdout=PIPE, stderr=STDOUT, bufsize=1, universal_newlines=True) as p:
        with open(log_out, 'w') as f:
            for line in p.stdout:
                print(line, end='') # process line here
                f.write(line)

    if p.returncode != 0:
        raise CalledProcessError(p.returncode, p.args)


def download_schemas(download_path, pds4_version, dev_release=False):
    pds4_version_short = convert_pds4_version_to_alpha(pds4_version)
    try:
        if not dev_release:
            base_url = PDS_SCHEMA_URL
        else:
            base_url = PDS_DEV_SCHEMA_URL

        fname = 'PDS4_PDS_' + pds4_version_short
        for suffix in ['.xsd', '.sch']:
            logger.info(f'Downloading {base_url + fname + suffix}')
            r = requests.get(base_url + fname + suffix, allow_redirects=True)
            with open(os.path.join(download_path, fname + suffix), 'wb') as f:
                f.write(r.content)

    except Exception as e:
        logger.error('Failure attempting to download schemas.')
        raise e
        traceback.print_exc
        sys.exit(1)


def main():

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
        description=__doc__)
    parser.add_argument('--deploy_dir',
                        help='directory to deploy the validate tool on the file system',
                        default='/tmp')
    parser.add_argument('--token',
                        help='github token')
    parser.add_argument('--schemas',
                        help='path(s) to schemas to validate against')
    parser.add_argument('--schematrons',
                        help='path(s) to schematrons to validate against')
    parser.add_argument('--skip_content_validation',
                        help='validate: skip content validation',
                        action='store_true', default=False)
    parser.add_argument('--failure_expected', dest='failure_expected',
                        help='validate expected to fail',
                        action='store_true', default=False)
    parser.add_argument('--datapath',
                        help='path(s) to data to validate',
                        required=True)
    parser.add_argument('--output_log_path',
                        help='path(s) to output validate run log file',
                        default=os.path.join('tmp', 'logs'))
    parser.add_argument('--with_pds4_version',
                        help=('force the following PDS4 version. software will '
                              'download and validate with this version of the '
                              'PDS4 Information Model. this version should be '
                              'the semantic numbered version. e.g. 1.14.0.0'))
    parser.add_argument('--development_release',
                        help=('flag to indicate this should be tested with a '
                              'development release of the PDS4 Standard.'),
                        action='store_true', default=False)

    args = parser.parse_args()

    token = args.token or os.environ.get('GITHUB_TOKEN')

    if not token:
        logger.error(f'Github token must be provided or set as environment variable (GITHUB_TOKEN).')
        sys.exit(1)

    try:
        validate_args = ['-R', 'pds4.label']

        if args.skip_content_validation:
            validate_args.append('--skip-content-validation')

        schemas = []
        if args.schemas:
            schemas.extend(glob.glob(args.schemas, recursive=True))

        schematrons = []
        if args.schematrons:
            # validate_args.append('-S')
            schematrons.extend(glob.glob(args.schematrons, recursive=True))

        if args.development_release:
            if not args.with_pds4_version:
                raise argparse.ArgumentError('--with_pds4_version must be specified when using --development_release')

        if args.with_pds4_version:
            download_schemas(DOWNLOAD_PATH, args.with_pds4_version, dev_release=args.development_release)
            schemas.extend(glob.glob(os.path.join(DOWNLOAD_PATH, '*.xsd')))
            schematrons.extend(glob.glob(os.path.join(DOWNLOAD_PATH, '*.sch')))

        if schemas:
            validate_args.append('-x')
            validate_args.extend(schemas)

        if schematrons:
            validate_args.append('-S')
            validate_args.extend(schematrons)

        validate_args.append('-t')
        validate_args.extend(glob.glob(args.datapath, recursive=True))


        pkg = download_asset(get_latest_release(token), args.deploy_dir, file_extension='.zip')
        sw_dir = unzip_asset(pkg, args.deploy_dir)

        exec_validate(os.path.join(sw_dir, 'bin', 'validate'), validate_args, log_path=args.output_log_path)
    except CalledProcessError:
        if not args.failure_expected:
            logger.error(f'FAILED: Validate failed unexpectedly. See output logs.')
            sys.exit(1)
    except Exception as e:
        traceback.print_exc()
        sys.exit(1)

    logger.info(f'SUCCESS: Validation complete.')

if __name__ == '__main__':
    main()



import unittest
import os
import logging
from pds_github_util.branches.git_actions import ping_repo_branch

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

GITHUB_TOKEN = os.environ.get("GITHUB_TOKEN")

class MyTestCase(unittest.TestCase):
    def test_ping_branch(self):
        ping_repo_branch('NASA-PDS/pdsen-corral', 'gh-pages', 'test commit message', token=GITHUB_TOKEN)


if __name__ == '__main__':
    unittest.main()



import logging
import requests
from bs4 import BeautifulSoup
import github3
from packaging import version

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

VOID_URL = 'https://en.wikipedia.org/wiki/Void_(astronomy)'

def is_dev_version(tag_name):
    return tag_name.endswith("-dev") or tag_name.endswith("-SNAPSHOT")


def get_max_tag(tag_name, other_tag_name ):
    vers = version.parse(tag_name)
    other_vers = version.parse(other_tag_name)
    return tag_name if (vers > other_vers) else other_tag_name


class CattleHead():

    _icon_dict = {
        'download': 'https://nasa-pds.github.io/pdsen-corral/images/download.png',
        'manual': 'https://nasa-pds.github.io/pdsen-corral/images/manual.png',
        'changelog': 'https://nasa-pds.github.io/pdsen-corral/images/changelog.png',
        'requirements': 'https://nasa-pds.github.io/pdsen-corral/images/requirements.png',
        'license' : 'https://nasa-pds.github.io/pdsen-corral/images/license.png',
        'feedback': 'https://nasa-pds.github.io/pdsen-corral/images/feedback.png'
    }

    def __init__(self, name, github_path, version=None, dev=False, token=None):
        logger.info(f'create cattleHead {name}, {github_path}')
        self._name = name
        self._github_path = github_path
        self._org = self._github_path.split("/")[-2]
        self._repo_name = self._github_path.split("/")[-1]
        self._token = token
        gh = github3.login(token=self._token)
        self._repo = gh.repository(self._org, self._repo_name)
        self._description = self._repo.description
        self._changelog_url = f"http://{self._org}.github.io/{self._repo_name}/pdsen-corral/CHANGELOG.html"
        self._changelog_signets = self._get_changelog_signet()
        self._dev = dev
        self._version = self._get_latest_patch(minor=version)

    def _get_latest_patch(self, minor=None):

        latest_tag = None
        for tag in self._repo.tags():
            if is_dev_version(tag.name) and self._dev:  # if we have a dev version and we look for dev version
                latest_tag = get_max_tag(tag.name, latest_tag) if latest_tag else tag.name
            elif not (is_dev_version(tag.name) or self._dev):  # if we don't have a dev version and we look for stable version
                if minor is None \
                        or (minor and (tag.name.startswith(minor) or tag.name.startswith(f'v{minor}'))):
                    latest_tag = get_max_tag(tag.name, latest_tag) if latest_tag else tag.name

        return latest_tag.__str__() if latest_tag else None

    def _get_cell(self, function):
        link_func = eval(f'self._get_{function}_link()')
        return f'[![{function}]({self._icon_dict[function]})]({link_func} "{function}")' if link_func else ' '

    def _get_download_link(self):
        return f'{self._github_path}/releases/tag/{self._version}'

    def _get_manual_link(self):
        if self._version:
            url = f'https://{self._org}.github.io/{self._repo_name}/{self._version}'
            if requests.get(url).status_code != 404:
                return url
            elif self._version[0] == 'v':
                url = f'https://{self._org}.github.io/{self._repo_name}/{self._version[1:]}'
                if requests.get(url).status_code != 404:
                    return url

        return f'https://{self._org}.github.io/{self._repo_name}'


    def _get_changelog_link(self):
        if self._version:
            if self._version in self._changelog_signets:
                return self._changelog_signets[self._version]
            else:
                return None
        else:
            return "https://www.gnupg.org/gph/en/manual/r1943.html"

    def _get_requirements_link(self):
        url = f'https://{self._org}.github.io/{self._repo_name}/pdsen-corral/{self._version}/REQUIREMENTS.html'
        logger.info(f'try url {url} for requirements')
        if self._version and requests.get(url).status_code != 404:
            return url
        else:
            return None

    def _get_license_link(self):
        return f'https://raw.githubusercontent.com/NASA-PDS/{self._repo_name}/master/LICENSE.txt'

    def _get_feedback_link(self):
        return f'{self._github_path}/issues/new/choose'

    def get_table_row(self):
        icon_cells = [self._get_cell(k) for k in self._icon_dict.keys()]

        more_description = f' [...]({self._github_path})' if len(self._description)>75 else ''

        return [self._name,
                self._version if self._version else "None",
                self._description[:75] + more_description,
                *icon_cells
        ]

    def _get_changelog_signet(self):
        headers = requests.utils.default_headers()
        changelog = requests.get(self._changelog_url, headers)
        soup = BeautifulSoup(changelog.content, 'html.parser')
        changelog_signets = {}
        for h2 in soup.find_all('h2'):
            changelog_signets[h2.find("a").text] = "#".join([self._changelog_url, h2.get('id')])

        return changelog_signets


